%GeislerAlbrecht.m
%
% Implementation of the ideal observer model to predict discrimmination
% thresholds and identification performance baesed on measured macaque V1
% properties as described in;
%
% Geisler, W.S., & Albrecht, D.G. (1997). Visual cortex neurons in monkeys
% and cats: detection, discrimination, and identification. Vis Neurosci, 14
% (5), 897-919.
%
% Written By G.M. Boynton, October 2008 at the University of Washington

%% Define a single V1 neuron's parameters
%parameters are for descriptive functions decribed on page 902.

pNeuron.K = 1.4;       %fanofactor (scale factor determining variance of response)
pNeuron.r0 = 1;        %baseline firing rate (spikes/sec)

%contrast
pNeuron.c50 = .1;      %semisaturation constant
pNeuron.n = 2;         %exponent
pNeuron.rMax = 9;      %max response (spikes/sec)

%spatial frequency
pNeuron.uc = 4;        %preferred spatial frequency (c/deg)
pNeuron.hu = 2;        %spatial frequency bandwithd (high)
pNeuron.lu = 1;        %spatial frequency bandwith (low)

%orientation
pNeuron.thetac = 90;   %preferred orientation 0 = vertical
pNeuron.btheta = 30;   %orientation bandwidth (degrees)

fontSize =12;
%% Plot neuron contrast response, orientation and spatial frequency tuning

%Contrast response function
pStim.c = 2.^(-6:.25:0);         %contrast
pStim.sf = 4;                    %spatial frequency (c/deg)
pStim.theta = 90;                %orientation (deg)

%Get the mean of the response
resp = getV1Response(pNeuron,pStim);

figure(1)
clf
subplot(3,3,1)
plot(100*pStim.c,resp);
xlabel('Contrast (%)','FontSize',fontSize);
ylabel('Response (spikes/sec)','FontSize',fontSize);
set(gca,'FontSize',fontSize);

%% Orientation tuning
pStim.c = 1;
pStim.sf = 4;
pStim.theta = linspace(0,180,101);

resp = getV1Response(pNeuron,pStim);

subplot(3,3,2)
plot(pStim.theta,resp);
set(gca,'XTick',0:45:180);
xlabel('Orientation (deg)','FontSize',fontSize);
set(gca,'FontSize',fontSize);

%% Spatial frequency tuning

pStim.c = 1;
pStim.sf = linspace(0,10,101);
pStim.theta = 90;
[resp,varResp] = getV1Response(pNeuron,pStim);

subplot(3,3,3)
plot(pStim.sf,resp);
set(gca,'XTick',0:2.5:10);
xlabel('SpatialFrequency  (c/deg)','FontSize',fontSize);
set(gca,'FontSize',fontSize);

%% Contrast discrimination thresholds (like figure 5a)

pStim.c = linspace(0,1,101);        %contrast
pStim.sf = 4;                       %spatial frequency
pStim.theta = 90;                   %orientation

varName = 'c';  %define the parameter to increment for threshold
thresh = getDiscriminationThreshold(pNeuron,pStim,varName);

subplot(3,3,4)
plot(100*pStim.c,100*thresh);
maxc = find(isnan(thresh),1,'first');  %only draw lower contrasts
set(gca,'XLim',100*[0,pStim.c(maxc)]);
xlabel('Contrast (%)','FontSize',fontSize);
ylabel('Increment threshold (%)','FontSize',fontSize);
set(gca,'FontSize',fontSize);
%% Orientation discrimination thresholds (like figure 5d)

pStim.c = 1;                        %contrast
pStim.sf = 4;                       %spatial frequency
pStim.theta = linspace(0,180,101);   %orientation
varName = 'theta';
thresh = getDiscriminationThreshold(pNeuron,pStim,varName);

subplot(3,3,5)
plot(pStim.theta,thresh);
xlabel('Orientation (deg)','FontSize',fontSize);
ylabel('Increment threshold (deg)','FontSize',fontSize);
set(gca,'FontSize',fontSize);
%% Spatial frequency discrimination thresholds (like figure 5c)

pStim.c = 1;                            %contrast
pStim.sf = linspace(0,10,101);          %spatial frequency
pStim.theta = 90;                       %orientation

varName = 'sf';
thresh = getDiscriminationThreshold(pNeuron,pStim,varName);

subplot(3,3,6)
plot(pStim.sf,thresh);
xlabel('Spatial frequency (c/deg)','FontSize',fontSize);
ylabel('Increment threshold (c/deg)','FontSize',fontSize);
set(gca,'FontSize',fontSize);
%% Identification performance for contrast (Figure 7a)

%optimal stimulus
pStim.c = 1;        %contrast
pStim.sf = 4;         %spatial frequency (to be defined later)
pStim.theta = 90;     %orientation

varName = 'c';
vals = linspace(0,1,501);

p= getIdentificationPerformance(pNeuron,pStim,varName,vals);
[limits,p95] = getConfidenceLimits(p,vals,0.95);

subplot(3,3,7)
patch([limits(1),vals(p>p95),limits(2)],[0,p(p>p95),0]/max(p),[.9,.9,1],'EdgeColor','none');
hold on
plot(vals,p/max(p))
plot([min(vals),max(vals)],p95*[1,1]/max(p),'r:');
xlabel('Contrast (%)','FontSize',fontSize)
ylabel('Relative probability','FontSize',fontSize)
set(gca,'FontSize',fontSize);
%% Identification performance for orientation (Figure 7d)

%optimal stimulus
pStim.c = 1;        %contrast
pStim.sf = 4;         %spatial frequency (to be defined later)
pStim.theta = 90;     %orientation

varName = 'theta';
vals = linspace(0,180,501);
p= getIdentificationPerformance(pNeuron,pStim,varName,vals);
[limits,p95] = getConfidenceLimits(p,vals,0.95);


subplot(3,3,8)
patch([limits(1),vals(p>p95),limits(2)],[0,p(p>p95),0]/max(p),[.9,.9,1],'EdgeColor','none');
hold on
plot(vals,p/max(p))
plot([min(vals),max(vals)],p95*[1,1]/max(p),'r:');
xlabel('Orientation (deg)','FontSize',fontSize)
set(gca,'FontSize',fontSize);

%% Identification performance for spatial Frequency (Figure 7c)

%optimal stimulus
pStim.c = 1;        %contrast
pStim.sf = 4;         %spatial frequency (to be defined later)
pStim.theta = 90;     %orientation

varName = 'sf';
vals = linspace(0,10,501);

%pStim.sf = 3;

p= getIdentificationPerformance(pNeuron,pStim,varName,vals);
[limits,p95] = getConfidenceLimits(p,vals,0.95);

subplot(3,3,9)
patch([limits(1),vals(p>p95),limits(2)],[0,p(p>p95),0]/max(p),[.9,.9,1],'EdgeColor','none');
hold on
plot(vals,p/max(p))
plot([min(vals),max(vals)],p95*[1,1]/max(p),'r:');
xlabel('Spatial frequency (c/deg)','FontSize',fontSize)
set(gca,'FontSize',fontSize);

%% Contrast and spatial frequency (see Figure 9)

pStim.c = linspace(0,1,101);        %contrast
pStim.sf = 4*ones(size(pStim.c));         %spatial frequency
pStim.theta = 90;     %orientation

varName = 'c';  %contrast
threshC = getDiscriminationThreshold(pNeuron,pStim,varName);

varName = 'sf';  %spatial frequency
threshSF = getDiscriminationThreshold(pNeuron,pStim,varName);

figure(2)
clf
haxes = plotyy(100*pStim.c,100*threshC,100*pStim.c,threshSF);

axes(haxes(1));
ylabel('Contrast threshold (%)','FontSize',fontSize);
set(gca,'FontSize',fontSize);

axes(haxes(2));
ylabel('Spatial Frequency threshold (c/deg)','FontSize',fontSize);
xlabel('Contrast (%)','FontSize',fontSize);
set(gca,'FontSize',fontSize);

%% Population Analysis

%sample parameters based on the distributions in Table 2 (monkey)
nNeurons = 1000;
pNeurons = sampleV1Neurons(nNeurons);
e=nNeurons*.5/100; %efficiency

%% Contrast discrimination performance for population

pStim.c = linspace(0,1,101);              %contrast
pStim.sf = 4*ones(size(pStim.c));         %spatial frequency
pStim.theta = 90;                         %orientation

varName = 'c';

%method 1: find best threshold for each neuron (Figure 10a)
bestThresh = zeros(1,nNeurons);
bestC = zeros(1,nNeurons);
thresh = zeros(nNeurons,length(pStim.c));
for i=1:nNeurons
    %pStim.sf = pNeurons(i).uc;
    %pStim.theta = pNeurons(i).thetac;
    thresh(i,:) = getDiscriminationThreshold(pNeurons(i),pStim,varName);

    [bestThresh(i),id] = min(thresh(i,:));
    bestC(i)= pStim.c(id);
end

figure(3)
clf
plot(pStim.c,thresh);
xlabel('Contrast (%)','FontSize',fontSize);
ylabel('Contrast threshold (%)','FontSize',fontSize);

figure(4)
clf
subplot(2,3,1);

plot(bestC,bestThresh,'ko','MarkerSize',3,'MarkerFaceColor','k');
xlabel('Contrast (%)','FontSize',fontSize);
ylabel('Lowest threshold','FontSize',fontSize);

%method 2: get d' pooling across the whole population
thresh = getDiscriminationThreshold(pNeurons,pStim,varName,5);
subplot(2,3,4)

plot(pStim.c,thresh);
xlabel('Contrast (%)','FontSize',fontSize);
ylabel('Contrast threshold (%)','FontSize',fontSize);

%% Spatial frequency discrimination performance for population

pStim.c = 1;        %contrast
pStim.sf = linspace(0,10,101);         %spatial frequency
pStim.theta = 90;     %orientation

varName = 'sf';

%method 1: find best threshold for each neuron (Figure 10a)
bestThresh = zeros(1,nNeurons);
bestSF = zeros(1,nNeurons);
thresh = zeros(nNeurons,length(pStim.sf));
for i=1:nNeurons

    %pStim.theta = pNeurons(i).thetac;
    thresh(i,:) = getDiscriminationThreshold(pNeurons(i),pStim,varName);

    [bestThresh(i),id] = min(thresh(i,:));
    bestSF(i)= pStim.sf(id);
end


subplot(2,3,2)
plot(bestSF,bestThresh,'ko','MarkerSize',3,'MarkerFaceColor','k');
xlabel('Spatial frequency (c/deg)','FontSize',fontSize);

%method 2: get d' for whole population

thresh = getDiscriminationThreshold(pNeurons,pStim,varName,e);
subplot(2,3,5)

plot(pStim.sf,thresh);
xlabel('Spatial frequency (c/deg)','FontSize',fontSize);

%% Orientation discrimination performance for population

pStim.c = 1;        %contrast
pStim.sf = 4;         %spatial frequency
pStim.theta = linspace(0,180,101);     %orientation

varName = 'theta';

%method 1: find best threshold for each neuron (Figure 10a)
bestThresh = zeros(1,nNeurons);
bestTheta = zeros(1,nNeurons);
thresh = zeros(nNeurons,length(pStim.theta));
for i=1:nNeurons

    %pStim.theta = pNeurons(i).thetac;
    thresh(i,:) = getDiscriminationThreshold(pNeurons(i),pStim,varName);

    [bestThresh(i),id] = min(thresh(i,:));
    bestTheta(i)= pStim.theta(id);
end


subplot(2,3,3)
plot(bestTheta,bestThresh,'ko','MarkerSize',3,'MarkerFaceColor','k');
xlabel('Orientation (deg)','FontSize',fontSize);

%method 2: get d' for whole population
thresh = getDiscriminationThreshold(pNeurons,pStim,varName,e);
subplot(2,3,6)

plot(pStim.theta,thresh);
xlabel('Orientation (deg)','FontSize',fontSize);

%% Mean V1 contrast-response function


pStim.c = linspace(0,1,101);        %contrast
pStim.sf = 4;         %spatial frequency
pStim.theta = 90;     %orientation

resp = zeros(nNeurons,length(pStim.c));

for i=1:nNeurons
    resp(i,:)= getV1Response(pNeurons(i),pStim);
end

popResp = mean(resp);

figure(5)
clf
subplot(1,2,1)

hold on
plot(100*pStim.c,resp,'-','Color',[.95,.95,1]);
plot(100*pStim.c,popResp,'k-','LineWidth',2);
ylim([min(popResp)*.9,max(popResp)*1.1]);
xlabel('Contrast (%)','FontSize',fontSize);
ylabel('Response','FontSize',fontSize);

subplot(1,2,2)

hold on
plot(log(100*pStim.c),resp,'-','Color',[.95,.95,1]);
plot(log(100*pStim.c),popResp,'k-','LineWidth',2);
ylim([min(popResp)*.9,max(popResp)*1.1]);
xlabel('Contrast (%)','FontSize',fontSize);
ylabel('Response','FontSize',fontSize);
logx2raw

%% Estimate contrast discrimination from this based on additive noise

baseVals = exp(linspace(log(.03),log(.6),21));

low = zeros(size(baseVals));
high = .9*ones(size(baseVals));

baseResp = interp1(pStim.c,popResp,baseVals);
inc= .025;
for i=1:21
    mid = (low+high)/2;
    incResp = interp1(pStim.c,popResp,baseVals+mid);
    incResp(isnan(incResp)) = 1;
    deltaR = incResp-baseResp;
    high(deltaR>inc) = mid(deltaR>inc);
    low(deltaR<inc) = mid(deltaR<inc);
end
thresh =    (low+high)/2;
figure(6)
clf
plot(log(100*baseVals),log(100*thresh),'b-');
logx2raw
logy2raw



%% show an image of how the neurons 'tile' the stimulus space

[sf,theta] = meshgrid(linspace(0,10,101),linspace(0,180,101));

pStim.c = 1;
pStim.sf = sf;
pStim.theta = theta;
img = zeros(size(sf));


for i=1:nNeurons
    resp= getV1Response(pNeurons(i),pStim);
    %img = img+resp;
end

figure(6)
clf
imagesc(sf(1,:),theta(:,1),img);
xlabel('Spatial frequency (c/deg)','FontSize',fontSize);
ylabel('Orientation (deg)','FontSize',fontSize);
colormap(gray);






