%HeegerNormalizationModel.m

%Implements the 'contrast normalization model' of V1 neurons from Heeger, D.J.
%(1992). Normalization of cell responses in cat striate cortex. Vis Neurosci, 9 (2), 181-197.
%
%Note: in this implementation, linear receptive fields and stimuli are
%represented as single frames, rather than time-varying.  The experiments
%that are modeled in this paper do not involve temporal factors per se, so
%this is a valid approximation.

%The matlab function 'NormalizationModel.m' can handle 3D RFs and stimuli,
%but the computations take much, much longer and aren't any different.

%Written by G.M. Boynton at the University of Washington on 11/9/08

%% Set up 'world' parameters
fontSize =12;

%set up 'world' parameters
pWorld.n = [50,50,1];      %[ny, nx, nt] in pixels
pWorld.visAngle = 4;        %horizontal extent of visual angle
pWorld.dur = 0;             %total stimulus duration (seconds)
pWorld.background = 0;      %background color: [-1,1]

pWorld = initWorld(pWorld); %get the x and y meshrid matrices

%% Model Parameters

%Half wave rectification and nonlinearity parameters
pModel.k = 500;         %scale factor
pModel.Vrest = 0;       %threshold
pModel.p = 2;           %exponent

%Normalization parameters
pModel.sig = .5;        %semisaturation constant
pModel.a = 1;           %scale factor

%% Create a population of spatio-temporal frequency filters
clear pRF RF

sfList = [.8,1,1.2];        %list of preferred spatial frequencies
ctList = [1,1,1];           %list of scale factors for these sf filters
angList = 0:45:179;         %list of preferred orientations
phaseList = [0,90,180,270]; %Quadrature phases

%spatial parameters for Gabor receptive field
pRF.center = [0,0];         %[0,0] is center of image
pRF.sf = NaN;               %'carrier' spatial frequency (c/deg) (to be varied)
pRF.sig = NaN;              %Gaussian standard deviation (to be varied with sf)
pRF.ang = NaN;              %orientation (deg) (to be varied)
pRF.phase = 0;              %0 = sin phase
pRF.contrast = NaN;           %scale factor (to be varied with sf)
pRF.nt = 1;                 %number of temporal frames

%Generate a cell array containing the receptive fields by calling 'makeGaborRF'
%Columns (1st dimension) will be orientation
%Rows (2nd dimension0 will be spatial frequency
%Third dimension are the four phases (Quadrature phase)
for i=1:length(sfList)
    pRF.sf = sfList(i);         %spatial frequency
    pRF.sig = .33/pRF.sf;       %width is inversely proportional to sf
    pRF.contrast = ctList(i);   %contrast
    for j=1:length(angList)
        pRF.ang = angList(j);   %orientation
        for k=1:4  %quadrature phases
            pRF.phase = phaseList(k);
            RF{i,j,k} = makeGaborRF(pWorld,pRF);
        end
    end
end

%Determine which cell to use for plotting
ci = find(sfList == 1);         %Spatial frequency index of the cell we'll be plotting (1 c/deg)
cj = find(angList == 90);       %Orientation index of the cell we'll be plotting (horizontal)


%% show the RFs

figure(1)
clf
count  =1;
for i=1:length(sfList)
    for j=1:length(angList)
        subplot(length(sfList),length(angList),count)
        if i==ci & j==cj
            hold on
            patch(pWorld.n(2)*[-.05,-.05,1.05,1.05],pWorld.n(1)*[-.05,1.05,1.05,-.05],'y','EdgeColor','none');
        end
        h=image((RF{i,j,1}+1)*127.5)
        axis equal
        axis off
        count = count+1;
    end
end

cmap = zeros(256,3);
cmap(1:128,1) = linspace(1,0,128).^.6;
cmap(129:256,2) = linspace(0,1,128);
colormap(cmap);

%% Contrast response function and adapation:
%Figure 2 in the paper, based on Ohzawa et al. (1985)
%Shows contrast response functions for varying values of pModel.sig, the
%semisaturation constant as a model for adaptation - contrast gain.

ctList = 2.^(-5:.5:0);              %list of contrasts
sigList = [.03,.0625,.125,.25,.5];  %list of semisaturation constants

%Define the grating stimulus
pStim.type = 'grating';
pStim.sf = 1;                       %spatial frequency (c/deg)
pStim.ang = 90;                     %orientation (90 is horizontal)
pStim.phase = 90;                   %spatial phase
pStim.contrast = NaN;               %contrast [0-1] (to be varied)
pStim.p0 = [0,0];                   %center

pModel.sig = NaN;                   %semisaturation constant (to be varied)

resp = zeros(length(ctList),length(sigList));
for i=1:length(ctList)
    pStim.contrast = ctList(i);
    stim = makeStim(pWorld,pStim);  %horizontally oriented grating
    for j=1:length(sigList)
        pModel.sig = sigList(j);    %semisaturation constant
        C = normalizationModel(pWorld,pModel,stim,RF);
        resp(i,j) = C(ci,cj);
    end
end

%Plot the results
figure(2)
clf
plot(log(ctList*100),resp,'.-','MarkerSize',20);
set(gca,'XTick',log(ctList*100));
xlabel('Contrast','FontSize',fontSize)
ylabel('Response','FontSize',fontSize)
logx2raw(exp(1),0);
legend(num2str(sigList'),'Location','SouthEast');
set(gca,'FontSize',fontSize);

%% Contrast response for different spatial frequencies
%figure 3 in the paper (based on Albrecht and Hamilton (1982).

ctList = 2.^(-5:0);             %list of contrasts
sfList = [.5,.707,1,1.414,2];     %list of spatial frequencies

pModel.sig = .03;                %semisaturation constant

resp = zeros(length(ctList),length(sfList));
for i=1:length(ctList)
    pStim.contrast = ctList(i);
    for j=1:length(sfList)
        pStim.sf = sfList(j);
        stim = makeStim(pWorld,pStim); %horizontally oriented grating
        C = normalizationModel(pWorld,pModel,stim,RF);
        resp(i,j) =C(ci,cj);
    end
end

%plot the results
figure(3)
clf
plot(log(ctList*100),resp,'.-','MarkerSize',20);
set(gca,'XTick',log(ctList*100));
xlabel('Contrast','FontSize',fontSize);
ylabel('Response','FontSize',fontSize)
ylim([0,max(resp(:))*1.1]);
logx2raw(exp(1),0);
legend(num2str(sfList'),'Location','NorthWest')
set(gca,'FontSize',fontSize);

%% Cross-orientation suppression (Figures 5 and 6)
%Based on Bonds (1989)
%Left side: orientation selectivity of a complex cell for a 10% contrast grating
%Right side: supressive effect of superimposing a second 10% contrast 'masking' 
%stimulus on the preferred grating.  Figures 4 and 5 are for semisaturation 
%constants of .03 and .17 respectively.

angList = 0:10:180;         %list of orientations
pStim.contrast = .1;        %list of contrasts
pStim.sf = 1;               %spatial frequency
pStim.ang = 90;             %orientation of base stimulus

sigList = [.03,.17];        %the two semisaturation constants

%Make the 'base' grating stimulus
baseStim = makeStim(pWorld,pStim); %horizontally oriented (preferred) grating

for sigNum = 1:length(sigList)
    pModel.sig = sigList(sigNum);

    resp = zeros(length(angList),2);
    for i=1:length(angList)
        pStim.ang = angList(i);
        maskStim = makeStim(pWorld,pStim);
        C = normalizationModel(pWorld,pModel,maskStim,RF);
        resp(i,1) =C(ci,cj);

        C = normalizationModel(pWorld,pModel,maskStim+baseStim,RF);
        resp(i,2) =C(ci,cj);
    end
    
    figure(3+sigNum)  %Orientation tuning
    clf
    subplot(1,2,1)
    plot(angList,resp(:,1),'.-','MarkerSize',20);
    set(gca,'XTick',angList(1:2:end));
    xlabel('Orientation (deg)','FontSize',fontSize);
    ylim([0,max(resp(:))*1.1]);

    subplot(1,2,2);  %Effect of adding masking stimulus
    plot(angList,resp(:,2),'.-','MarkerSize',20);
    set(gca,'XTick',angList(1:2:end));
    hold on
    plot([min(angList),max(angList)],max(resp(:,1)*[1,1]),'k:');
    xlabel('Orientation (deg)','FontSize',fontSize);
    ylim([0,max(resp(:))*1.1]);
    title(sprintf('Semisaturation: %5.2f',pModel.sig));
end

%% Spatial Frequency Tuning (Figure 7)
%Based again on Bonds (1989)
%Effect of adding a second masking stimulus of a non-preferred orientation
%on a stimulus of preferred spatial frequency and orientation.
%Left side: spatial frequency selectivity of a complex cell for a 10% contrast grating
%Right side: supressive effect of superimposing a second 10% or 20% contrast 'masking' 
%stimulus on the preferred grating. 

sfList = 2.^[-5:.5:2];      %list of spatial frequencies
pStim.contrast = .1;        %contrast of base stimulus
pStim.sf = 1;               %spatial frequency of base stimulus
pStim.ang = 90;             %orientation of base stimulus

maskAng = 0;  %masking stimulus is off-orientation
maskCtList = [.1,.2];       %list of masking contrasts

pModel.sig = .12;            %Semisaturation constant: very dependent on this value!

%make the base stimulus
baseStim = makeStim(pWorld,pStim); %horizontally oriented (preferred) grating

resp = zeros(length(sfList),3);
for i=1:length(sfList)
    pStim.sf = sfList(i);
    stim = makeStim(pWorld,pStim);
    C = normalizationModel(pWorld,pModel,stim,RF);
    resp(i,1) =C(ci,cj);
end

pStim.ang = maskAng;
pStim.contrast = maskCtList(1);

for i=1:length(sfList)
    pStim.sf = sfList(i);
    stim = makeStim(pWorld,pStim);

    C = normalizationModel(pWorld,pModel,stim+baseStim,RF);
    resp(i,2) =C(ci,cj);
end
pStim.contrast = maskCtList(2);

for i=1:length(sfList)
    pStim.sf = sfList(i);
    stim = makeStim(pWorld,pStim);

    C = normalizationModel(pWorld,pModel,stim+baseStim,RF);
    resp(i,3) =C(ci,cj);
end

%Plot the results
figure(6) %spatial frequency tuning
clf
subplot(1,2,1)
plot(log(sfList),resp(:,1),'.-','MarkerSize',20);
set(gca,'XTick',log(2.^[-4:2:4]));
xlabel('Spatial frequency (c/deg)','FontSize',fontSize);
logx2raw(exp(1),1)
ylim([0,max(resp(:))*1.1]);



subplot(1,2,2);  %base + 10% and 20% contrast masks
plot(log(sfList),resp(:,2:3),'.-','MarkerSize',20);

hold on
plot([min(log(sfList)),max(log(sfList))],max(resp(:,1)*[1,1]),'k:');
ylim([0,max(resp(:))*1.1]);
set(gca,'XTick',log(2.^[-4:2:4]));
xlabel('Mask Spatial frequency (c/deg)','FontSize',fontSize);
logx2raw(exp(1),1);
legend({'10%','20%'});

%% Cross orientation suppression (Figure 8)
%based on Bonds (1989)
%
%Effect of off-oriented mask stimulus of various contrasts on contrast
%response at preferred orientation.

ctList = 2.^(-5:.5:0);
maskCtList= [0,.05,.1,.2];

pStim.contrast = NaN;
pStim.sf = 1;
pStim.ang = 90;

pModel.sig = .01;  %Very dependent on this value!

resp = zeros(length(ctList),length(maskCtList));

for i=1:length(ctList)
    %'base' grating
    pStim.ang = 90; %preferred orientation
    pStim.contrast= ctList(i);  %base contrast
    baseStim = makeStim(pWorld,pStim);
    for j=1:length(maskCtList)
        pStim.ang = 0;  %off-orientation
        pStim.contrast = maskCtList(j);  %mask contrast
        maskStim = makeStim(pWorld,pStim);
        C = normalizationModel(pWorld,pModel,baseStim+maskStim,RF);
        resp(i,j) =C(ci,cj);
    end
end

%plot the results
figure(7)
clf

plot(log(ctList*100),resp,'.-','MarkerSize',20);
set(gca,'XTick',log(ctList*100));
xlabel('Contrast','FontSize',fontSize);
ylim([0,max(resp(:))*1.1]);
logx2raw(exp(1),0);
legend(num2str(maskCtList'),'Location','SouthEast');


