% PlaidDemo.m
%
%   Demonstrates the perceived direction of the sum of two overlapping
%   moving gratings (a plaid), and shows their predicted direction of
%   motion based on their 'intersection of constraints'.
%
%   November 4, 2008, G.M. Boynton wrote it at the University of Washington

%% Define 'World' parameters
pWorld.n = [400,400,1];       % [ny, nx, nt] in pixels
pWorld.visAngle = 10;    	  %horizontal extent of visual angle
pWorld.dur = 0;               %duration (not relevant here, but needed for initWorld)
pWorld.background = 0;

pWorld = initWorld(pWorld);   %get the x and y meshrid matrices

%other graphics parameters
fontSize =12;
frameRate = 20;               %frame rate of presentation (Hz)
colList = {[1,0,0],[0,1,0]};  %two colors for slide bars and intersection of constraints figure
d = 10;                       %axis limits for intersection of contraints figure (deg/sec)
%% Initial parameters for the two gratings

pStim(1).ang = 45;
pStim(1).phase = 0;
pStim(1).sf = 1;
pStim(1).tf = 2;
pStim(1).p0 = [-1.5,0];
pStim(1).size = 3;
pStim(1).contrast = .5;

pStim(2).ang = 225;
pStim(2).phase = 0;
pStim(2).sf = 1;
pStim(2).tf = 2;
pStim(2).p0 = [1.5,0];
pStim(2).size = 3;
pStim(2).contrast = .5;

%% Prepare the figures

%zero out the stimulus image
stim = pWorld.background*ones(pWorld.n(1),pWorld.n(2));

%show it and capture the handle to the image
figure(1)
clf
c = image( (stim+1)*127);
colormap(gray(256))
axis equal
axis off

%% Set up the user controls

%list of fields for sliders
fields = {'ang','sf','tf','contrast','size'}';
%min and max values for sliders
mn = [0,0,0,0,0];
mx = [360,4,10,.5,10];
dx = .15;  %width of slider
dy = .05;  %y spacing of slider
for i=1:2
    x = (i-1)*(1-dx);  %x position of slider (between 0 and 1)
    y = 0.2;           %y position of slider
    for j=1:length(fields)
        %slider
        slider(i,j) = uicontrol('Units','normalized','Position',[x,y,dx,dy]);
        set(slider(i,j),'Style','Slider','String',fields{i});
        set(slider(i,j),'CallBack','somethingHappened=1;');
        set(slider(i,j),'Min',mn(j));
        set(slider(i,j),'Max',mx(j));
        set(slider(i,j),'BackgroundColor',colList{i});
        set(slider(i,j),'Value',getfield(pStim(i),fields{j}));
        set(slider(i,j),'SliderStep',[1/360,1/36]);

        %text above the slider
        textHandle(i,j) = uicontrol('Units','Normalized','Style','text','Position',[x,y+dy,dx,.04]);
        y=y+dy*2.5;
    end
end

%go and stop buttons
goButton = uicontrol('Units','normalized','Style','toggle','Position',[.3,.95,.2,.05],'String','Go','CallBack','PlaidDemo;','Value',1);
stopButton = uicontrol('Units','normalized','Style','toggle','Position',[.5,.95,.2,.05],'String','Stop','Value',0);

%% Animation loop

t=0;
somethingHappened=1;  %Set this to 1 so parameters are set on the first frame.
tic  %start the clock

while ~get(stopButton,'Value')  %Loop until stopButton is clicked

    if somethingHappened %The user moved a slider bar (or the program just started)
        %update the stimulus parameter and the text above the slider
        for i=1:2
            for j=1:length(fields)
                val = get(slider(i,j),'Value');
                pStim(i) = setfield(pStim(i),fields{j},val);
                textStr = sprintf('%s: %3.1f',fields{j},val);
                set(textHandle(i,j),'String',textStr);
            end
        end

        %(re)draw the 'intersection of constraints' in figure 2
        figure(2)
        clf
        hold on
        set(gcf,'Color',[.5,.5,.5]);    %figure background color
        set(gca,'Color',[.2,.2,.2]);           %graph (axis) background color

        for i=1:2  %loop for the two gratings

            %velocity vector
            v =pStim(i).tf/pStim(i).sf;      %velocity is tf/sf
            vx(i) = v*cos(pStim(i).ang*pi/180);  %x component of velocity
            vy(i) = -v*sin(pStim(i).ang*pi/180); %y component of velocity
        end

        %calculate the intersection (you can do the algebra...)
        xc = (vy(1)-vy(2)+vx(1)^2/vy(1)-vx(2)^2/vy(2))/(vx(1)/vy(1)-vx(2)/vy(2));
        yc = vy(1)-(vx(1)/vy(1))*(xc-vx(1));

        for i=1:2

            %draw vector from origin to (vx,vy)
            plot([0,vx(i)],[0,vy(i)],'-','Color',2*pStim(i).contrast*colList{i},'LineWidth',2);

            %draw the orthogonal line through (vx,vy)
            slope = -vx(i)/vy(i);  %orthogonal slope to velocity vector
            
            %find two extreme points on the line
            xx = [vx(i)-d,vx(i)+d];
            yy = [vy(i)-d*slope,vy(i)+d*slope];
            %draw this line with width 1
            plot(xx,yy,'-','Color',2*pStim(i).contrast*colList{i},'LineWidth',1);
            
            %draw part of the line to the intersection with width 2
            plot([vx(i),xc],[vy(i),yc],'-','Color',2*pStim(i).contrast*colList{i},'LineWidth',2);
        end

        %blue line between orgin and intersection
        plot([0,xc],[0,yc],'bo-','MarkerFaceColor','b');

        set(gca,'XLim',[-d,d]);
        set(gca,'YLim',[-d,d]);
        axis square
        xlabel('Vx (deg/sec)' ,'FontSize',fontSize);
        ylabel('Vy (deg/sec)','FontSize',fontSize);

        %Define the circular Apertures (only need to do this if something happened, not every frame)
        for i=1:2
            window{i} = (pWorld.x-pStim(i).p0(1)).^2 + (pWorld.y-pStim(i).p0(2)).^2 <= pStim(i).size^2;
        end
        somethingHappened = 0;  %reset this variable
    end

    %Generate the image
    stim = pWorld.background*ones(pWorld.n(1),pWorld.n(2));  %zero out the image
    %Add the two gratings to the image
    for i=1:2
        ramp = cos(pStim(i).ang*pi/180)*(pWorld.x-pStim(i).p0(1)) + (sin(pStim(i).ang*pi/180)*pWorld.y-pStim(i).p0(2));
        stim = stim + window{i}.*(pStim(i).contrast*sin(2*pi*pStim(i).sf*ramp-pStim(i).phase));
        %increment the phase for the next frame
        pStim(i).phase = pStim(i).phase + 2*pi*pStim(i).tf/frameRate;
    end

    %update the image
    set(c,'CData',(stim+1)*127);
    drawnow

    %Increment the time
    t=t+1/frameRate;
    while toc<t;
        %wait here until the clock has caught up to 't'
    end
end

%% 'Stop' button was hit, loop is over.  Turn off the 'Go' button
set(goButton,'Value',0);