%Priebe.m
%
%Generates figures from Priebe, N.J., & Ferster, D. (2008). Inhibition,
%spike threshold, and stimulus selectivity in primary visual cortex. Neuron, 57 (4), 482-497.
%% Define 'World' parameters
pWorld.n = [200,200,100];  %[ny, nx, nt] in pixels
pWorld.visAngle = 10;   %horizontal extent of visual angle
pWorld.dur = .5;        %duration (seconds)
pWorld.background = 0;

pWorld = initWorld(pWorld);  %get the x and y meshrid matrices

fontSize =12;

%% Define and plot Power law model converting membrane potential to spike rate
clear pModel
pModel.k = 1;
pModel.Vrest = .2;
pModel.p = 3;

Vm = linspace(0,1.5,101);
spikeRate = powerLawModel(pModel,Vm);

figure(2);
clf
subplot(1,2,1)
plot(Vm,spikeRate,'b-');
xlabel('Membrane Potential (Vm)','FontSize',fontSize);
ylabel('Firing Rate (spikes/sec)','FontSize',fontSize);

%% Simple cell RF (Gabor)
clear pRF
pRF.center = [0,0];
pRF.sf = .5;  %c/deg
pRF.sig = .75 %
pRF.ang = 0; %orientation (deg)
pRF.phase = -90; %0 = sin phase
pRF.contrast = 1;

RFGabor = makeGaborRF(pWorld,pRF);

figure(1)
clf
showRF(pWorld,RFGabor);

%% 1) Sharpening of orientation tuning (Figure 2)

%Rotating grating stimulus
pStim.type = 'grating';
pStim.sf = 0.5;
pStim.phase = -90;
pStim.contrast = .5;
pStim.p0 = [0,0];

nAng = pWorld.n(3);
angList = linspace(-90,90,nAng);

stimRot = zeros([pWorld.n(1),pWorld.n(2),nAng]);

for i=1:nAng
    pStim.ang = angList(i);
    stimRot(:,:,i) = makeStim(pWorld,pStim);

end

figure(1)
clf
showStimulus(pWorld,stimRot,RFGabor);

% Calculate the Linear response
Vm = linearResponse(pWorld,stimRot,RFGabor);

% Threshold and power law model converting membrane potential to spike

spikeRate = powerLawModel(pModel,Vm);

figure(2)
subplot(1,2,2)
haxes = plotyy(angList,Vm,angList,spikeRate);

xlabel('Angle of bar (deg)','FontSize',fontSize);
axes(haxes(1));
set(gca,'YLim',[0,max(Vm)]*1.1);

ylabel('Membrane potential (VmO)','FontSize',fontSize);
axes(haxes(2));
set(gca,'YLim',[0,max(spikeRate)]*1.1);
ylabel('Firing Rate (Hz)','FontSize',fontSize);

set(gcf,'PaperPosition',[1,1,5,4]);

%% 2) Cross-orientation Supression  (Figure 3)

%Define four LGN RFs (DOG's with different centers
%

clear RF pRF

cList = linspace(-.5,1,4);

pRF(1).sig = [.6963,.7433];
pRF(1).contrast = [18,16];

RF = zeros(pWorld.n(1),pWorld.n(2),4);
for i=1:4
    pRF(i) = pRF(1);
    pRF(i).center = [0,cList(i)];
    RF(:,:,i) = makeDOGRF(pWorld,pRF(i));
end

allRF = sum(RF,3)/4;

% Moving grating stimulus

pStim.type = 'DriftingGrating';
pStim.sf = 0.5;
pStim.ang = 90;
pStim.phase = 90;
pStim.contrast = .32;
pStim.p0 = [0,0];
pStim.tf = 4;
stimH = makeStim(pWorld,pStim); %horizontally oriented grating

pStim.ang = 0;
pStim.contrast = .32;  %use .08 for Figures D, E and F
stimV = makeStim(pWorld,pStim); %vertically oriented grating
stimP = stimH+stimV;  %plaid

%linear model

pModel(1).k = 50;
pModel(1).Vrest = -1;
pModel(1).p = 1;

%nonlinear model

pModel(2).k = 100;
pModel(2).Vrest = 0; %rectification
pModel(2).p = .5;   %compressive nonlinearity

figure(3)
clf


for modelNum = 1:2
    % Calculate the Linear response

    RespH = zeros(pWorld.n(3),4);
    RespV = zeros(pWorld.n(3),4);
    RespP = zeros(pWorld.n(3),4);

    for i=1:4
        VmH = linearResponse(pWorld,stimH,RF(:,:,i));
        RespH(:,i)  = powerLawModel(pModel(modelNum),VmH);
        VmV = linearResponse(pWorld,stimV,RF(:,:,i));
        RespV(:,i)   = powerLawModel(pModel(modelNum),VmV);
        VmP = linearResponse(pWorld,stimP,RF(:,:,i));
        RespP(:,i)  = powerLawModel(pModel(modelNum),VmP);
    end


    subplot(3,2,modelNum)
    plot(pWorld.t,RespH);
    hold on
    plot(pWorld.t,mean(RespH,2),'k-','LineWidth',2);
    set(gca,'XLim',[0,pWorld.dur]);
    set(gca,'YLim',[0,100]);
    xlabel('Time (sec)','FontSize',fontSize);
    ylabel('Resp','FontSize',fontSize);

    if modelNum == 1
        title('Linear Model');
    else
        title('Nonlinear Model');
    end

    subplot(3,2,modelNum+2)
    plot(pWorld.t,RespV);
    hold on
    plot(pWorld.t,mean(RespV,2),'k-','LineWidth',2);
    set(gca,'XLim',[0,pWorld.dur]);
    set(gca,'YLim',[0,100]);
    xlabel('Time (sec)','FontSize',fontSize);
    ylabel('Resp','FontSize',fontSize);

    subplot(3,2,modelNum+4)
    plot(pWorld.t,RespP);
    hold on
    plot(pWorld.t,mean(RespP,2),'k-','LineWidth',2);
    set(gca,'XLim',[0,pWorld.dur]);
    set(gca,'YLim',[0,100]);
    xlabel('Time (sec)','FontSize',fontSize);
    ylabel('Resp','FontSize',fontSize);
end

%% 3) Contrast Invariance (Figure 4)

cList = [.25,.5,1];     %list of contrasts

clear pModel

%model 1: threshold and w/ linear rise
pModel(1).k = 50;
pModel(1).Vrest = .05;
pModel(1).p = 1;

%model 2: threshold w/ quadratic rise
pModel(2).k = 50;
pModel(2).Vrest = 0.05;
pModel(2).p = 2;

figure(4)
clf

for modelNum = 1:2

    Resp = zeros(pWorld.n(3),length(cList));
    Vm = zeros(pWorld.n(3),length(cList));

    for cNum = 1:length(cList)
        %Linear response of Gabor to rotating grating (as above)
        Vm(:,cNum) =  linearResponse(pWorld,cList(cNum)*stimRot,RFGabor);
        %Model function from Vm to spikeRate
        Resp(:,cNum)   = powerLawModel(pModel(modelNum),Vm(:,cNum));
    end

    subplot(2,2,modelNum*2-1);
    plot(angList,Vm);
    hold on
    plot([-90,90],pModel(modelNum).Vrest*[1,1],'k:');
    xlabel('Orientation (deg');
    ylabel('Response');
  
    subplot(2,2,modelNum*2)
    plot(angList,Resp);
    xlabel('Orientation (deg');
    ylabel('Response');
end











