function resp = linearResponse(pWorld,stim,RF);
%
%resp = linearResponse(pWorld,stim,RF);
%
%Calculates the linear response of the stimulus sequence 'stim' and the
%receptive field 'RF'.  RF can be a 3D matrix containing a 3D space-time
%filter.
%
%Parameters:
%
%   stim                3D matrix of size pWorld.n containing stimulus images over
%                       time in 3rd dimension.  -1 <= stim <= 1.
%
%   RF                  3D matrix of size [pWorld.n(1),pWorld.n(2),<n>]
%                       containing linear receptive field images over time in 3rd
%                       dimension.  -1< = RF <= 1.
%
%   resp                Output vector of length pWorld.n(3).

%10/7/08 Written by G.M. Boynton at the University of Washington

%space and time pixel steps (used for scaling for pixels size)
dx = pWorld.x(1,2) - pWorld.x(1,1);
dy = pWorld.y(2,1) - pWorld.y(1,1);
if length(pWorld.t)>1
    dt = pWorld.t(2) - pWorld.t(1);
else
    dt = 1;
end

%reshape stim and RFs to unwrap space into a single dimension
tmpStim = double(reshape(stim,pWorld.n(1)*pWorld.n(2),pWorld.n(3)))';
tmpRF = reshape(RF,pWorld.n(1)*pWorld.n(2),size(RF,3))';

switch ndims(RF)
    case 2  %RF is a single frame, so just use the dot product (faster).
        resp = tmpStim*tmpRF';  %dot product
        resp = resp*dx*dy;      %scaling by pixels size
    case 3  %RF is a 3D space-time filter, so we need to use convolution
        resp = zeros(pWorld.n(3),pWorld.n(1)*pWorld.n(2));
        for i=1:pWorld.n(1)*pWorld.n(2);  %convolve across time for each pixel in space
            tmp = conv(tmpStim(:,i),tmpRF(:,i));
            resp(:,i) = tmp(1:pWorld.n(3));
        end
        %Then sum across space (and scale)
        resp = sum(resp,2)*dx*dy;
end