%% LinearModelDemo.m
%
%Demonstrates the output of the linear model for a variety of receptive
%fields (difference of Gaussians, Gabors, Hubel-Wiesel model) and a variety
%of stimuli (spots, gratings, bars).

%Written October 2008 by Geoffrey Boynton at the Unversity of Washington

%set up 'world' parameters
pWorld.n = [200,200,101];   %[ny, nx, nt] in pixels4
pWorld.visAngle = 10;       %horizontal extent of visual angle
pWorld.dur = 5;             %total stimulus duration (seconds)
pWorld.background = 0;      %background color: [-1,1]

pWorld = initWorld(pWorld); %get the x and y meshrid matrices

%% LGN Receptive field: Modeled as a difference of Gaussians (DOG)

%Define the DOG parameters 
clear pRF RF
pRF.center = [0,0];
pRF.sig = [.6963,.7433];
pRF.contrast = [9,8];

%Make it and show it
RF = makeDOGRF(pWorld,pRF);
figure(1)
showRF(pWorld,RF);

%Show the middle slice profile
figure(2)
clf
plot(pWorld.x(1,:),RF(:,end/2))
set(gca,'FontSize',16);
set(gcf,'PaperPosition',[1,1,3,2]);

%% Generate a stimulus that's a spot of increasing size

clear pStim
pStim.type = 'spot';
pStim.p0 = [0,0];
pStim.contrast = 1;

nWidth = pWorld.n(3);
widthList = linspace(0,5,nWidth);

stim = zeros(pWorld.n(1),pWorld.n(2),nWidth);

rsq = (pWorld.x(:,:,1)-pStim.p0(1)).^2 + (pWorld.y(:,:,1)-pStim.p0(2)).^2;
for i=1:length(widthList)
    stim(:,:,i) = rsq<=widthList(i)^2;
end
figure(1)
showStimulus(pWorld,stim,RF);

%% Linear response of DOG to growing spot - demonstrating size tuning

resp = linearResponse(pWorld,stim,RF);

figure(2);
clf
plot(widthList,resp)
xLabel('Width of spot (deg)','FontSize',16);
yLabel('Response','FontSize',16);
set(gca,'FontSize',16);
set(gcf,'PaperPosition',[1,1,7,5]);

%% Linear response to DOG and drifting grating - sinusoidal response
%Note: A linear system always responds to a sinusoid with a sinusoid of the
%same frequency (just a different phase and amplitude).  So this sinusoidal
%output isn't unique to the DOG.

clear pStim
%Define the parameters
pStim.type ='DriftingGrating';
pStim.sf = 0.5;      %c/deg
pStim.tf = 1;       %Hz
pStim.ang = 0;       %0 degrees is vertical
pStim.phase = 0;     %starting phase
pStim.p0 = [0,0];    %starting position
pStim.contrast = 1;  %between 0 and 1;
driftingGrating = makeStim(pWorld,pStim);

figure(1)
showStimulus(pWorld,driftingGrating,RF);

resp = linearResponse(pWorld,drifingGrating,RF);
figure(2)
clf
plot(pWorld.t,resp);


xLabel('Time (sec)','FontSize',16);
set(gca,'XLim',[0,pWorld.dur]);
yLabel('Response','FontSize',16);
set(gca,'FontSize',16);
set(gcf,'PaperPosition',[1,1,5,4]);

%% V1 receptive fields: modeled by an oriented Gabor function

clear pRf
%define the Gabor parameters
pRF.center = [0,0];
pRF.sf = .5;            %c/deg
pRF.sig = [2,1];        %deg (major and minor axes)  if sig(2) > sig(1) it's long and skinny
pRF.ang = 0;            %orientation (deg)
pRF.phase = -90;        %0 = sin phase, -90 is excitatory in the middle
pRF.contrast = 1;       

RF = makeGaborRF(pWorld,pRF);

figure(1)
showRF(pWorld,RF);
set(gcf,'PaperPosition',[1,1,4,4]);

%% The linear response of a Gabor RF to a moving bar stimulus.
% Note how it follows the excitatory and inhibitory flanks of the RF

clear pStim
pStim.type = 'bar';
pStim.ang =0;
pStim.length = 20;
pStim.width = .5;
pStim.contrast = 1;
pStim.dir = 90;
pStim.p0 = [-pWorld.visAngle/2,0];
pStim.v = pWorld.visAngle/pWorld.dur;  %deg/sec

movingBar = makeStim(pWorld,pStim);

figure(1)
showStimulus(pWorld,movingBar,RF);

resp = linearResponse(pWorld,movingBar,RF);


figure(2)
clf
plot(pWorld.t,resp);
set(gca,'XLim',[0,max(pWorld.t)]);

xLabel('Time (sec)','FontSize',16);
yLabel('Response','FontSize',16);
set(gca,'FontSize',16);
set(gcf,'PaperPosition',[1,1,5,4]);

%% Orientation tuning: Linear Gabor filter and a Rotating Bar Stimulus

pStim.p0 = [0,0];
pWorld.n(3) = 101;
angList = linspace(-90,90,pWorld.n(3));
pWorld.dur = 5;
pWorld = initWorld(pWorld);  %get the x and y meshrid matrices

nFrames = pWorld.n(3);
pWorld.n(3) =1;

rotBar = zeros([pWorld.n(1),pWorld.n(2),nFrames]);

%Generate the rotating bar stimulus frame by frame
for i=1:nFrames
    pStim.ang = angList(i);
    tempStim = makeStim(pWorld,pStim);
    rotBar(:,:,i) = tempStim(:,:,1);
end

pWorld.n(3) = nFrames;

figure(1)
showStimulus(pWorld,rotBar,RF);

resp = linearResponse(pWorld,rotBar,RF);

figure(2)
plot(angList,resp);
set(gca,'XTick',-90:45:90);
xLabel('Angle of bar (deg)','FontSize',16);
yLabel('Response','FontSize',16);
set(gca,'FontSize',16);
set(gcf,'PaperPosition',[1,1,5,4]);



%% The Gabor receptive field and spatial frequency selectivity
%Has maximum respons at the 'carrier' frequency of the Gabor

%Create a spatial frequency 'sweep' stimulus:

sfList = linspace(0,2,pWorld.n(3));

freqSweep = zeros(pWorld.n);

for i=1:pWorld.n(3)
    freqSweep(:,:,i) = cos(2*pi*pWorld.x*sfList(i));
end

figure(1)
showStimulus(pWorld,freqSweep,RF);

resp = linearResponse(pWorld,freqSweep,RF);

figure(2)
clf
plot(sfList,resp);


xLabel('Spatial frequency (c/deg)','FontSize',16);
yLabel('Response','FontSize',16);
set(gca,'FontSize',16);
set(gcf,'PaperPosition',[1,1,5,4]);

%% The Gabor RF and Drifting Grating
%
%Guess what? It's a sinusoid again.



figure(1)
showStimulus(pWorld,driftingGrating,RF);

resp = linearResponse(pWorld,driftingGrating,RF);

figure(2)
clf
plot(pWorld.t,resp);


xLabel('Time (sec)','FontSize',16);
set(gca,'XLim',[0,pWorld.dur]);
yLabel('Response','FontSize',16);
set(gca,'FontSize',16);
set(gcf,'PaperPosition',[1,1,5,4]);



%% Hubel Wiesel model
%
%Generate 3 DOG RF's in a row and them together.  
%
%Note: because it's a linear system, we can either add the RF's first
%before applying the stimulus, or measure the responses to the stimulus for
%each of the 3 DOG's and add the responses.  You'll get the same thing.

%Gaussian #1 (top)

clear RF
pRF.center = [0,-1];
pRF.sig = [.6963,.7433];
pRF.contrast = [9,8];

RF1 = makeDOGRF(pWorld,pRF);

%  #2 (center)
pRF.center = [0,0];
RF2 = makeDOGRF(pWorld,pRF);

% #3 (bottom)
pRF.center = [0,1];
RF3 = makeDOGRF(pWorld,pRF);

% All 3 - add the RFs together

RF = RF1 + RF2 + RF3;

figure(1)
showRF(pWorld,RF);


%% Hubel Wiesel and Rotating Bar Stimulus

resp = linearResponse(pWorld,rotBar,RF);

figure(2)
clf
plot(angList,resp);
set(gca,'XTick',-90:45:90);
xLabel('Angle of bar (deg)','FontSize',16);
yLabel('Response','FontSize',16);
set(gca,'FontSize',16);
set(gcf,'PaperPosition',[1,1,4,3]);


%% Compare orientation tuning of Hubel and Wieslel RF to a well-matched
% Gabor RF

pRF.center = [0,0];
pRF.sf = .5;  %c/deg
pRF.sig = [2,1]; %deg (major and minor axes)
pRF.ang = 0; %orientation (deg)
pRF.phase = -90; %0 = sin phase
pRF.contrast = 1;

RF = makeGaborRF(pWorld,pRF);

resp = linearResponse(pWorld,rotBar,RF);


figure(3)
clf
plot(angList,resp);
set(gca,'XTick',-90:45:90);
xLabel('Angle of bar (deg)','FontSize',16);
yLabel('Response','FontSize',16);
set(gca,'FontSize',16);
set(gcf,'PaperPosition',[1,1,4,3]);

%% Just for fine, make an image tiling Gabor RFs across the visual field!

nGabors = 120;

RF = zeros(pWorld.n(1),pWorld.n(2));

mx = max(pWorld.x(:));
my = max(pWorld.y(:));
sepSq = .75^2;

clear x y
for i=1:nGabors
    minDsq = 0;
    while minDsq<sepSq
        x(i) = 2*(rand(1)-.5)*mx;
        y(i) = 2*(rand(1)-.5)*my;

        dsq = (x(1:(i-1))-x(i)).^2 + (y(1:(i-1))-y(i)).^2;
        minDsq = min(dsq);
    end

    pRF.center = [x(i),y(i)];
    pRF.ang = rand(1)*90;
    pRF.sf = rand(1)*5;
    pRF.sig = rand(1,2)*.15+.1;
    pRF.phase = 2*pi*rand(1);
    pRF.contrast = 1;
    RF = RF+ makeGaborRF(pWorld,pRF);
end
figure(1)
showRF(pWorld,RF);



