%ModelFest.m

%% Set up 'world' parameters
fontSize =12;

%set up 'world' parameters
pWorld.n = [256,256,1];      %[ny, nx, nt] in pixels
pWorld.visAngle = 2.133;        %horizontal extent of visual angle
pWorld.dur = 0;             %total stimulus duration (seconds)
pWorld.background = 0;      %background color: [-1,1]

pWorld = initWorld(pWorld); %get the x and y meshrid matrices

%% load in the ModelFest thresholds and images

%subjectNames modelFestThresh modelFestSEM
load ModelFestThresholds
% Calculate the means
meanThresh = mean(thresh,2)';

%ModelFestStimuli stim stimName
load ModelFestStimuli stim stimName

%Reshape to make image of all stimuli for threshold plot
img = reshape(stim,256,256*43);

% Convert images to contrast
stim = (double(stim)-128)/127;

%% Show the stimuli

nw = 8;
nh = 6;
figure(1)
clf

x = 0;
y = 1-1/nh;
k=.8;
for i=1:43
    subplot('Position',[x,y,k/nw,k/nh]);
    image( (stim(:,:,i)+1)*128);
    colormap(gray(256));
    axis square
    axis off
    x = x+1/nw;
    if x>.9
        x=0;
        y=y-1/nh;
    end
    text(10,230,num2str(i));
end

set(gcf,'PaperPosition',[1,1,10,7.5]);
%% Figure 1: plot the thresholds (in decibels) under images of the stimuli
figure(2)
clf
subplot('Position',[.1,.9,.85,.9/43])
image(img);
colormap(gray(256))
axis off

hdata = subplot('Position',[.1,.1,.85,.8]);
colList= hsv(size(thresh,2)+1);

for i=1:size(thresh,2)
    hold on
    plot(-20*thresh(:,i),'o','Color',colList(i,:),'MarkerSize',5,'MarkerFaceColor',colList(i,:))
end
set(gca,'XLim',[.5,43.5]);
hold on

ylim = get(gca,'YLim');
for i=.5:44
    plot(i*[1,1],ylim,'-','Color',[.8,.8,.8]);
end
%logy2raw
xlabel('Stimulus number','FontSize',fontSize);
ylabel('Threshold (dB)','FontSize',fontSize);
set(gcf,'PaperPosition',[1,1,10,7.5]);


%% Fit a Gaussian function to the CSF based on the 1st 10 thresholds

%Spatial frequencies of first 10 stimuli
sf = [1.12,2,2.83,4,5.66,8,11.3,16,22.6,30];
%Sensitivities (1/thresholds) for these 10 stimuli
csf = 10.^meanThresh(1:10);

%plot the threshold data as a CSF
figure(3)
clf
h1=plot(log(sf),csf,'bo-','MarkerFaceColor','b');
logx2raw

%fit the CSF with a Gaussian
pGauss.mean = 1;
pGauss.sig = 1;
pGauss.a = 1;

pGauss= fit('fitGaussian',pGauss,{'mean','sig','a'},sf,csf);

%Plot the fit on top of the real CSF
[err,Gaussian] = fitGaussian(pGauss,sf,csf);
hold on
h2=plot(log(sf),Gaussian,'rs-','MarkerFaceColor','r');
xlabel('Spatial frequency (c/deg)','FontSize',fontSize);
ylabel('Sensitivity','FontSize',fontSize);
legend([h1,h2],{'Data','Gaussian'},'FontSize',fontSize);
set(gca,'FontSize',fontSize);




%% Create a 2D filter (RF) based on the Gaussian CSF

%n is the 'Nyquist limit' (highest representable spatial frequency frequency)
pixperdeg = pWorld.n(1)/pWorld.visAngle;  %pixels per degree of visual angle
n = round(pixperdeg/2);

%these are the spatial frequencies associated with the FFT
sx = pWorld.n(2)/2;
sy = pWorld.n(1)/2;

%create a matrix of spatial frequencies
[wx,wy] = meshgrid(linspace(0,n,sx),linspace(0,n,sy));
wr = sqrt(wx.^2+wy.^2);
%create the filter by interplating the csf.

quadfilt = interp1(sf,csf,wr);

%quadfilt  = pGauss.a*exp(-(log(wr)-pGauss.mean).^2/pGauss.sig^2);
quadfilt(isnan(quadfilt)) = 0;
%fill in the inverse fft filter by filling all four quadrants
fftfilt = zeros(pWorld.n(1),pWorld.n(2));
fftfilt(1:sy,1:sx) = quadfilt;
fftfilt((end-sy+2):end,1:sx) = flipud(quadfilt(2:end,:));
fftfilt(1:sy,(end-sx+2):end) = fliplr(quadfilt(:,2:end));
fftfilt((end-sy+2):end,(end-sx+2):end) = flipud(fliplr(quadfilt(2:end,2:end)));

filt = fftshift(ifft2(fftfilt));
%filt = filt*15000;  %arbitrary scale value

figure(4)
showRF(pWorld,filt/max(filt(:)));
xlim(.2*[-1,1]);
set(gca,'YLim',.2*[-1,1]);

figure(5)

plotFFT(pWorld,filt,5,10);

%% Calculate the response to each stimulus by this filter
clear resp

respStim = zeros(size(sf));
for i=1:length(sf)
    pStim.sf = sf(i);
    respStim(i) = linearResponse(pWorld,stim(:,:,i),filt);
end

scFac =max(csf)/max(respStim);

figure(5)
clf
plot(log(sf),csf,'bo-','MarkerFaceColor','b')
hold on
plot(log(sf),scFac*respStim,'rs-','MarkerFaceColor','r')
set(gca,'XTick',log(sf));
logx2raw

%% Watson's 'Energy' Model predictions

pModel.beta = 2;  %pooling parameter (Minkowski pooling).  2 = 'energy'

%pull out only the center part of the filter for faster processing
id = abs(pWorld.x(1,:))<.4;
pModel.filt = filt(id,id);
pModel.inc = 2;

dx = pWorld.x(1,2)-pWorld.x(1,1);

%Convolve stimulous with filter
pModel.convImg = zeros(size(stim));
for i=1:size(stim,3)
    pModel.convImg(:,:,i) = conv2(stim(:,:,i),pModel.filt,'same')*dx*dx;
end

%% find the thresholds

pModel = fit('fitWatsonModel',pModel,{'inc'},thresh);

[RMS,predThresh] = fitWatsonModel(pModel,thresh);


%% Try just the linear response model
pModel.Vrest =0;
pModel.p = 1;
pModel.k= 1;

%pModel = fit('fitLinearModel',pModel,{'inc'},pWorld,stim,filt,thresh);
%[RMS,predThresh] = fitLinearModel(pModel,pWorld,stim,filt,thresh);


RMS

%% Plot the predictions over the thresholds in figure 1

figure(2)
subplot(hdata)
hold on
plot(20*log10(predThresh),'k-','LineWidth',2);

%% Look at the deviations

err = abs(-20*log10(predThresh)-20*meanThresh);

[foo,id] = sort(err,'descend');

figure(6)
nw = 8;
nh = 6;

clf
x = 0;
y = 1-1/nh;
k=.8;
for i=1:43
    subplot('Position',[x,y,k/nw,k/nh]);
    image( (stim(:,:,id(i))+1)*128);
    colormap(gray(256));
    axis square
    axis off
    x = x+1/nw;
    if x>.9
        x=0;
        y=y-1/nh;
    end
    text(10,230,sprintf('%d, %5.2f',id(i),err(id(i))));
end
set(gcf,'PaperPosition',[1,1,10,7.5]);

%% Look at ModelFest stimuli in the Fourier domain.

RF = fftshift(fftfilt);
RF = RF/max(RF(:));


clf
for i=1:43
    figure(i+6)
    clf
    w=plotFFT(pWorld,stim(:,:,i),1,5)
    title(num2str(i))
    hold on
    [c,h]=contour(w,w,RF,1/exp(1));
    set(h,'Color','b','LineWidth',2);
    set(gcf,'PaperPosition',[1,1,4,2]);
end










