%PlaidDemo.m
%
%Demonstrates the perceived direction of the sum of two overlapping
%gratings (plaids), and shows their predicted direction based on the
%'intersection of constraints' model.
%
%November 2008, G.M. Boynton wrote it at the University of Washington

%% Define 'World' parameters
pWorld.n = [200,200,1];  %[ny, nx, nt] in pixels
pWorld.visAngle = 10;   %horizontal extent of visual angle
pWorld.dur = (pWorld.n(3)-1)/1000;        %duration (seconds)
pWorld.background = 0;

pWorld = initWorld(pWorld);  %get the x and y meshrid matrices

fontSize =12;
frameRate = 40;  %frame rate of presentation (Hz)
%% Initial parameters for the two gratings

pStim(1).ang = 135;
pStim(1).phase = 0;
pStim(1).sf = 1;
pStim(1).tf = 3;
pStim(1).p0 = [-1.5,0];
pStim(1).size = 3;
pStim(1).contrast = .5;

pStim(2).ang = 45;
pStim(2).phase = 0;
pStim(2).sf = 1;
pStim(2).tf = 3;
pStim(2).p0 = [1.5,0];
pStim(2).size = 3;
pStim(2).contrast = 0;

%% Prepare the figures

%zero out the stimulus image
stim = pWorld.background*ones(pWorld.n(1),pWorld.n(2));

%show it and capture the handle to the image
figure(1)
clf
c = image( (stim+1)*127);
colormap(gray(256))
axis equal
axis off



%% Set up the user controls

%list of fields for sliders
fields = {'ang','sf','tf','contrast','size'}';
%min and max values for sliders
mn = [0,0,0,0,0];
mx = [360,4,10,.5,10];
dx = .15;
dy = .05;
for i=1:2
    x = (i-1)*(1-dx);
    y = 0.2;
    for j=1:length(fields)
        %slider
        slider(i,j) = uicontrol('Units','normalized','Position',[x,y,dx,dy],'Style','Slider','String',fields{i});
        set(slider(i,j),'CallBack','change=1;');
        set(slider(i,j),'Min',mn(j));
        set(slider(i,j),'Max',mx(j));
        %text above the slider

        textHandle(i,j) = uicontrol('Units','Normalized','Style','text','Position',[x,y+dy,dx,.04]);
        set(slider(i,j),'Value',getfield(pStim(i),fields{j}));

        y=y+dy*2.5;
    end
end

%quit button
quitButton = uicontrol('Units','normalized','Style','toggle','Position',[.4,.95,.2,.05],'String','Quit','Value',0);
change=1;  %change we can believe in.

%% Animation loop
tic
t=0;

%start the animation loop.  Quit the loop when the quitButton is hit.
while ~get(quitButton,'Value')
    
    %Generate the gratings
    stim = pWorld.background*ones(pWorld.n(1),pWorld.n(2));
    
    %Apertures
    for i=1:2
        window{i} = (pWorld.x-pStim(i).p0(1)).^2 + (pWorld.y-pStim(i).p0(2)).^2 <= pStim(i).size^2;
    end

    %Gratings
    for i=1:2  
        ramp = cos(pStim(i).ang*pi/180)*(pWorld.x-pStim(i).p0(1)) + (sin(pStim(i).ang*pi/180)*pWorld.y-pStim(i).p0(2));
        stim = stim + window{i}.*(pStim(i).contrast*sin(2*pi*pStim(i).sf*ramp-pStim(i).phase));
        pStim(i).phase = pStim(i).phase + 2*pi*pStim(i).tf/frameRate;
    end
    
    %update the image
    set(c,'CData',(stim+1)*127);
    drawnow

    %timing
    t=t+1/frameRate;
    while toc<t;
        %wait here until the clock has caught up to 't'
    end

    if change %something happened: user moved a slider bar
        %update the stimulus parameter and the text above the slider
        for i=1:2
            for j=1:length(fields)
                val = get(slider(i,j),'Value');
                pStim(i) = setfield(pStim(i),fields{j},val);
                textStr = sprintf('%s: %3.1f',fields{j},val);
                set(textHandle(i,j),'String',textStr);
            end
        end

        %(re) draw the 'intersection of constraints' figure
        figure(2)
        clf
        hold on
        set(gca,'Color','k');
        set(gcf,'Color',[.5,.5,.5]);
        for i=1:2
            if pStim(i).contrast>0
                col = [0,0,0];
                col(i) = 1;
                v =pStim(i).tf/pStim(i).sf;
                x = v*cos(pStim(i).ang*pi/180);
                y = -v*sin(pStim(i).ang*pi/180);

                plot([0,x],[0,y],'-','Color',2*pStim(i).contrast*col);
                %plot(x,y,'o','MarkerFaceColor',2*pStim(i).contrast*[1,1,1]);
                slope = -x/y;
                xx = [x-100,x+100];
                yy = [y-100*slope,y+100*slope];
                plot(xx,yy,'-','Color',2*pStim(i).contrast*col);
                plot(0,0,'wo','MarkerFaceColor','w');
            end
        end
        set(gca,'XLim',[-10,10]);
        set(gca,'YLim',[-10,10]);
        axis square
        xlabel('Vx (deg/sec)' ,'FontSize',8);
        ylabel('Vy (deg/sec)','FontSize',8);
    end
    change = 0;  %get ready for the next change.
end