function [resp,varResp] = getV1Response(pNeuron,pStim)
%[resp,varResp] = getV1Response(pNeuron,pStim)
%
%Calculates the mean and variance in response of a neuron defined by
%pNeuron to a stimulus defined by pStim based on descriptive functions as
%described by:
%   Geisler, W.S., & Albrecht, D.G. (1997). Visual cortex neurons in monkeys
%   and cats: detection, discrimination, and identification. Vis Neurosci, 14
%   (5), 897-919.
%
%V1 responses are modeled as a separable function of contrast, spatial
%frequency and orientation.  The paper also includes the parameters of
%spatial position, direction and temporal frequency, but these are left out
%of this model to keep things simple(r).
%
%Relevant fields:
%pNeuron 
%          K: %fanofactor (proportional constant for variance re mean
%         r0: %baseline response
%        c50: %semisaturation constant
%          n: %exponent for contrast-response
%       rMax: %max response
%         uc: %center spatial frequency (c/deg)
%         hu: %upper width of spatial frequency tuning
%         lu: %lower width of spatial frequency tuning
%     thetac: %center orientation tuning (deg)
%     btheta: %width of orientation tuning (deg)
% 
% pStim 
%         c:  %contrast
%        sf:  %spatial frequency
%     theta:  %orientation
%
%If any of the stimulus field parameters are vectors, then  the response is
%calculated for all values.  If more than one field is a vector, then these
%vectors must be the same length, indicating a set of stimuli that vary in
%more than one dimension.

%Written by G.M. Boynton on October 11, 2008 at the University of Washington

%contrast
rc = pStim.c.^pNeuron.n./(pStim.c.^pNeuron.n + pNeuron.c50.^pNeuron.n);

%spatial frequency
if isfield(pNeuron,'uc')
    ru = zeros(size(pStim.sf));
    sfLow = pStim.sf <= pNeuron.uc;
    sfHigh = pStim.sf > pNeuron.uc;

    ru(sfLow) = exp(-log(2)*( (pStim.sf(sfLow)-pNeuron.uc).^2/pNeuron.lu^2));
    ru(sfHigh) =  exp(-log(2)*( (pStim.sf(sfHigh)-pNeuron.uc).^2/pNeuron.hu^2));
else
    ru = 1;
end

%orientation
if isfield(pNeuron,'thetac')
    rtheta = exp(-log(2)*( (pStim.theta-pNeuron.thetac).^2/pNeuron.btheta^2));
else
    rtheta = 1;
end

%response is a separable function of response to contrast, orientation and
%spatial frequency:
r = rc.*ru.*rtheta;

resp = pNeuron.rMax*r+pNeuron.r0;
varResp = resp*pNeuron.K;
