function [C,S] = normalizationModel(pWorld,pModel,stim,RF,poolingRule,number)
%function [C,S] = normalizationModel(pWorld,,pModel,stim,RF,[poolingRule],[quickExp])
%
%Implements Heeger's (1991) contrast normalization model.
%
%Inputs:
%   pWorld             structure of 'world coordinates'
%
%   pModel             structure of model parameters:
%                      'Half-squaring' parameters:
%     pModel.k         scale factor
%     pModel.Vrest     threshold
%     pModel.p         exponent
%                      Normalization parameters:
%     pModel.sig       semisaturation constant
%     pModel.a         scale factor
%
%   stim               3D matrix containing image slices over time
%   RF                 3D matrix containing cell array of RFs
%
%   poolingRule        options for pooling across the array of neurons
%         'all'        provide responses to all RFs (default)
%         'mean'       calculate average of responses
%         'quick'      use 'quick' pooling with quickExp as a power
%
%Outputs:
%  C                   complex cell responses
%  S                   simple cell responses
%
%Note: if stim and RF are 3D matrices, then C and S will be [mxnxt] holding
%the response over time.  If they're 2D then they're single values (and the
%calculation is much, much faster).

%unwrap matric of RFs (e.g. sf x orientation) into one long vector
sz = size(RF);
RF = reshape(RF,prod(sz(1:end-1)),4);
nCells = size(RF,1);

%calculate the linear response and the half-squared response for each RF
clear A E S C
for i=1:nCells
    for j=1:4  %phases
        L = linearResponse(pWorld,stim,RF{i,j});
        A(i,j,:) = powerLawModel(pModel,L);
    end
    E(i,:) = mean(A(i,:,:),2);  %'Energy' output is mean of A's
end

%Average E's (energy) across spatial frequencies and orientations (all cells)
D = squeeze(mean(E,1));  %note: Heeger says 'sum', but that makes
%the result depend on the number of cells in the
%pool.

%Calculate contrast normalized response (both Single and Complex Cells)
for i=1:nCells  %loop through all cells
    for j=1:4  %loop through quadrature phases
        %simple cell: divide A by energy + semisaturation constant
        S{i}(j,:) = squeeze(pModel.a*A(i,j,:))'./(pModel.sig^2 + D);
    end
    %complex cell: divide E by energy + semisaturation constant
    C(i,:) = pModel.a*E(i,:)./(pModel.sig^2 + D);
end

%reshape simple and complex responses to original dimensions
S = reshape(S,sz(1:end-1));
C = reshape(C,[sz(1:end-1),size(stim,3)]);

%pooling rule

if ~exist('poolingRule','var')
    poolingRule = 'all';
end

switch poolingRule
    case 'mean'
        C = mean(C(:));
    case 'quick'
        C = (sum(C(:).^number))^(1/number);
    case 'index'
        C = C(number(1),number(2));
end

